import {
  Directive,
  Input,
  Output,
} from "@angular/core";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {ApplicationRolePermissionEnum} from "@app/shared/enums/application-role-permission.enum";
import {ApplicationRoleEnum} from "@app/shared/enums/application-role.enum";
import {LanguagesEnum} from "@app/shared/enums/languages.enum";
import {
  AppRoutesEnum,
  OrderRoutesEnum,
  RoutesEnum,
} from "@app/shared/enums/routes.enum";
import {ThemeEnum} from "@app/shared/enums/theme.enum";
import {PermissionUtil} from "@app/shared/utils/permission.util";
import {environment} from "@environments/environment";
import {UserExtended} from "@shared/models/business/user-extended.model";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  OAuth2Service,
  ObservableUtil,
  TRANSLATE,
  urlSeparator,
} from "solidify-frontend";

@Directive()
export abstract class AbstractMainToolbarPresentational extends SharedAbstractPresentational {
  @Input()
  logged: boolean = false;

  @Input()
  currentPath: string;

  @Input()
  currentLanguage: LanguagesEnum;

  @Input()
  user: UserExtended;

  institutionUrl: string = environment.institutionUrl;

  @Input()
  numberArchiveInCart: number = 0;

  @Input()
  numberMyOrderReadyNew: number = 0;

  @Input()
  numberPendingRequestNotificationInbox: number = 0;

  @Input()
  private userRoles: ApplicationRoleEnum[];

  // Theme
  private readonly _themeBS: BehaviorSubject<ThemeEnum | undefined> = new BehaviorSubject<ThemeEnum | undefined>(undefined);

  @Output("themeChange")
  readonly themeObs: Observable<ThemeEnum | undefined> = ObservableUtil.asObservable(this._themeBS);

  isProduction: boolean = environment.production;
  isDemoMode: boolean = environment.isDemoMode;
  isOpen: boolean = false;

  private get _theme(): ThemeEnum | undefined {
    return this._themeBS.getValue();
  }

  private set _theme(value: ThemeEnum | undefined) {
    this._themeBS.next(value);
  }

  get theme(): ThemeEnum | undefined {
    return this._theme;
  }

  @Input()
  set theme(theme: ThemeEnum | undefined) {
    this._theme = theme;
  }

  // Navigate
  private readonly _navigateBS: BehaviorSubject<string | undefined> = new BehaviorSubject<string | undefined>(undefined);
  @Output("navigateChange")
  readonly navigateObs: Observable<string | undefined> = ObservableUtil.asObservable(this._navigateBS);

  // Language
  private readonly _languageBS: BehaviorSubject<LanguagesEnum | undefined> = new BehaviorSubject<LanguagesEnum | undefined>(undefined);
  @Output("languageChange")
  readonly languageObs: Observable<LanguagesEnum | undefined> = ObservableUtil.asObservable(this._languageBS);

  // Logout
  private readonly _logoutBS: BehaviorSubject<void | undefined> = new BehaviorSubject<void | undefined>(undefined);
  @Output("logoutChange")
  readonly logoutObs: Observable<void | undefined> = ObservableUtil.asObservable(this._logoutBS);

  // Cart
  private readonly _cartBS: BehaviorSubject<void | undefined> = new BehaviorSubject<void | undefined>(undefined);
  @Output("cartChange")
  readonly cartObs: Observable<void | undefined> = ObservableUtil.asObservable(this._cartBS);

  private listMenus: Menu[] = [
    {
      click: (menu) => this.navigate(menu.path),
      path: RoutesEnum.homePage,
      labelToTranslate: TRANSLATE("app.toolbar.home"),
      isVisible: () => true,
      icon: "home",
    },
    {
      click: (menu) => this.openOrder(),
      path: RoutesEnum.order,
      labelToTranslate: TRANSLATE("app.toolbar.order"),
      isVisible: () => PermissionUtil.isUserHavePermission(this.logged, ApplicationRolePermissionEnum.userPermission, this.userRoles),
      icon: "cash-register",
      badgeCounter: () => "" + this.numberMyOrderReadyNew,
      badgeDescription: "Counter of new order ready in my order",
      badgeHidden: () => this.numberMyOrderReadyNew === 0,
    },
    {
      click: (menu) => this.navigate(menu.path),
      path: RoutesEnum.deposit,
      labelToTranslate: TRANSLATE("app.toolbar.deposit"),
      isVisible: () => PermissionUtil.isUserHavePermission(this.logged, ApplicationRolePermissionEnum.userPermission, this.userRoles),
      icon: "file-upload",
    },
    {
      click: (menu) => this.navigate(menu.path),
      path: RoutesEnum.preservationSpace,
      labelToTranslate: TRANSLATE("app.toolbar.preservation-space"),
      isVisible: () => PermissionUtil.isUserHavePermission(this.logged, ApplicationRolePermissionEnum.userPermission, this.userRoles),
      icon: "users",
      badgeCounter: () => "" + this.numberPendingRequestNotificationInbox,
      badgeDescription: "Counter of pending request notification in inbox",
      badgeHidden: () => this.numberPendingRequestNotificationInbox === 0,
    },
    {
      click: (menu) => this.navigate(menu.path),
      path: RoutesEnum.preservation,
      labelToTranslate: TRANSLATE("app.toolbar.preservation"),
      isVisible: () => PermissionUtil.isUserHavePermission(this.logged, ApplicationRolePermissionEnum.adminPermission, this.userRoles),
      icon: "boxes",
    },
    {
      click: (menu) => this.navigate(menu.path),
      path: RoutesEnum.admin,
      labelToTranslate: TRANSLATE("app.toolbar.admin"),
      isVisible: () => PermissionUtil.isUserHavePermission(this.logged, ApplicationRolePermissionEnum.adminPermission, this.userRoles),
      icon: "cogs",
    },
  ];

  get routesEnum(): typeof RoutesEnum {
    return RoutesEnum;
  }

  protected constructor(private oauthService: OAuth2Service) {
    super();
  }

  getListMenu(): Menu[] {
    return this.listMenus.filter(m => m.isVisible());
  }

  navigate(path: string): void {
    this._navigateBS.next(path);
  }

  useLanguage(language: LanguagesEnum): void {
    this._languageBS.next(language);
  }

  toggleMenu(): void {
    this.isOpen = !this.isOpen;
  }

  openCart(): void {
    this._cartBS.next();
  }

  openOrder(): void {
    if (this.userRoles.includes(ApplicationRoleEnum.admin) || this.userRoles.includes(ApplicationRoleEnum.root)) {
      this._navigateBS.next(AppRoutesEnum.order);
    } else {
      this._navigateBS.next(AppRoutesEnum.order + urlSeparator + OrderRoutesEnum.myOrder);
    }
  }

  login(): void {
    this.oauthService.initAuthorizationCodeFlow();
  }

  logout(): void {
    this._logoutBS.next();
  }

  navigateToInstitution(): void {
    window.open(this.institutionUrl, "_blank");
  }
}

interface Menu {
  path?: string;
  click: (menu: Menu) => void;
  labelToTranslate: string;
  icon: string;
  isVisible: () => boolean;
  badgeCounter?: (() => string) | undefined;
  badgeDescription?: string | undefined;
  badgeHidden?: (() => boolean) | undefined;
}
